package Util "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Util.rml
 ** module:      Util
 ** description: Miscellanous RML utilities
 **
 ** RCS: $Id: Util.rml 1798 2005-05-31 06:32:26Z petar $
 ** 
 ** This module contains various RML utilities *sigh*, mosly 
 ** related to lists.
 ** It is used pretty much everywhere. The difference between this 
 ** module and the ModUtil module is that ModUtil contains modelica 
 ** related utilities. The Util module only contains \"low-level\" 
 ** rml utilities, for example finding elements in lists.
 ** 
 ** This modules contains many functions that uses 'type variables' in RML.
 ** A type variable is exactly what it sounds like, a type bound to a variable.
 ** It is used for higher order functions, i.e. in RML the possibility to pass a 
 ** \"pointer\" to a function into another function. But it can also be used for 
 ** generic data types, like in  C++ templates.

 ** A type variable in RML is written as 'a
 ** For instance,
 ** function list_fill ('a,int) => 'a list
 ** the type variable 'a is here used as a generic type for the function list_fill, 
 ** which returns a list of n elements of a certain type.
 *"

public function list_fill "* function: list_fill
 ** Returns a list of n elements of type 'a.
 ** For example,
 ** list_fill(\"foo\",3) => {\"foo\",\"foo\",\"foo\"}
*"
  input Type_a in_type_a;
  input Integer in_integer;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_type_a,in_integer)
    local
      Type_a a;
      Integer n_1,n;
      VType_aList res;
    case (a,1) then {a}; 
    case (a,n)
      equation 
        n_1 = n - 1;
        res = list_fill(a, n_1); then a :: res;
  end matchcontinue;
end list_fill;

public function list_first "* function: list_first 
 ** Returns the first element of a list
 ** For example,
 ** list_first({3,5,7,11,13}) => 3
 "
  input VType_aList in_vtype_alist;
  output Type_a out_type_a;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_type_a:=
  matchcontinue (in_vtype_alist)
    local Type_a x;
    case (x :: _) then x; 
  end matchcontinue;
end list_first;

public function list_rest "* function: list_rest
 ** Returns the rest of a list.
 ** For example,
 ** list_rest({3,5,7,11,13}) => {5,7,11,13}
 *"
  input VType_aList in_vtype_alist;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_alist)
    local VType_aList x;
    case (_ :: x) then x; 
  end matchcontinue;
end list_rest;

public function list_last "* function: list_last
 ** Returns the last element of a list. If the list is the empty list, the function 
 ** fails.
 ** For example,
 ** list_last({3,5,7,11,13}) => 13
 ** list_last({}) => fail
 *"
  input VType_aList in_vtype_alist;
  output Type_a out_type_a;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_type_a:=
  matchcontinue (in_vtype_alist)
    local
      Type_a a;
      VType_aList rest;
    case {a} then a; 
    case (_ :: rest)
      equation 
        a = list_last(rest); then a;
  end matchcontinue;
end list_last;

public function list_flatten "* function: list_flatten
 ** Takes a list of lists and flattens it out, producing one list of all 
 ** elements of the sublists.
 ** For example
 ** list_flatten({ {1,2},{3,4,5},{6},{} }) => {1,2,3,4,5,6}
 *"
  input VType_aListList in_vtype_alistlist;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  type VType_aListList = list<VType_aList>;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_alistlist)
    local
      VType_aList r_1,l,f;
      VType_aListList r;
    case {} then {}; 
    case f :: r
      equation 
        r_1 = list_flatten(r);
        l = list_append(f, r_1); then l;
  end matchcontinue;
end list_flatten;

public function list_append_elt "* function: list_append_elt
 ** This function adds an element last to the list
 ** For example
 ** list_append_elt(1,{2,3}) => {2,3,1}
 *"
  input Type_a in_type_a;
  input VType_aList in_vtype_alist;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_type_a,in_vtype_alist)
    local
      Type_a elt,x;
      VType_aList xs_1,xs;
    case (elt,{}) then {elt}; 
    case (elt,x :: xs)
      equation 
        xs_1 = list_append_elt(elt, xs); then x :: xs_1;
  end matchcontinue;
end list_append_elt;

public function list_map "* function: list_map
 ** Takes a list and a function over the elements of the lists, which is applied
 ** for each element, producing a new list.
 ** For example
 ** list_map({1,2,3}, int_string) => { \"1\", \"2\", \"3\"}
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  output VType_bList out_vtype_blist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    output Type_b out_type_b;
  protected 
    replaceable type Type_b;
  end FuncType1;
  replaceable type Type_b;
  type VType_bList = list<Type_b>;
algorithm 
  out_vtype_blist:=
  matchcontinue (in_vtype_alist,in_functype1)
    local
      Type_b f_1;
      VType_bList r_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
    case ({},_) then {}; 
    case (f :: r,fn)
      equation 
        f_1 = fn(f);
        r_1 = list_map(r, fn); then f_1 :: r_1;
  end matchcontinue;
end list_map;

public function list_map__2 "* function list_map__2
 ** Takes a list and a function over the elements returning a tuple of two types,
 ** which is applied for each element producing two new lists.
 ** For example
 ** function split_real_string (real) => (string,string)  returns the string value at 
 ** each side of the decimal point.
 ** list_map__2({1.5,2.01,3.1415}, split_real_string) => ({\"1\",\"2\",\"3\"},{\"5\",\"01\",\"1415\"})
*"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  output VType_bList out_vtype_blist;
  output VType_cList out_vtype_clist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    output Type_b out_type_b;
    output Type_c out_type_c;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
  end FuncType1;
  replaceable type Type_b;
  type VType_bList = list<Type_b>;
  replaceable type Type_c;
  type VType_cList = list<Type_c>;
algorithm 
  (out_vtype_blist,out_vtype_clist):=
  matchcontinue (in_vtype_alist,in_functype1)
    local
      Type_b f1_1;
      Type_c f2_1;
      VType_bList r1_1;
      VType_cList r2_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
    case ({},_) then ({},{}); 
    case (f :: r,fn)
      equation 
        (f1_1,f2_1) = fn(f);
        (r1_1,r2_1) = list_map__2(r, fn); then (f1_1 :: r1_1,f2_1 :: r2_1);
  end matchcontinue;
end list_map__2;

public function list_map_1 "* function list_map_1
 ** Takes a list and a function over the list plus an extra argument sent to the function.
 ** The function produces a new value which is used for creating a new list.
 ** For example,
 ** list_map_1({1,2,3},int_add,2) => {3,4,5}
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  output VType_cList out_vtype_clist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    output Type_c out_type_c;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
  end FuncType1;
  replaceable type Type_b;
  replaceable type Type_c;
  type VType_cList = list<Type_c>;
algorithm 
  out_vtype_clist:=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b)
    local
      Type_c f_1;
      VType_cList r_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
      Type_b extraarg;
    case ({},_,_) then {}; 
    case (f :: r,fn,extraarg)
      equation 
        f_1 = fn(f, extraarg);
        r_1 = list_map_1(r, fn, extraarg); then f_1 :: r_1;
  end matchcontinue;
end list_map_1;

public function list_map_1r "* function list_map_1r
 ** Same as list_map_1 but swapped arguments on function.
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  output VType_cList out_vtype_clist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_b in_type_b;
    input Type_a in_type_a;
    output Type_c out_type_c;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
  end FuncType1;
  replaceable type Type_b;
  replaceable type Type_c;
  type VType_cList = list<Type_c>;
algorithm 
  out_vtype_clist:=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b)
    local
      Type_c f_1;
      VType_cList r_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
      Type_b extraarg;
    case ({},_,_) then {}; 
    case (f :: r,fn,extraarg)
      equation 
        f_1 = fn(extraarg, f);
        r_1 = list_map_1r(r, fn, extraarg); then f_1 :: r_1;
  end matchcontinue;
end list_map_1r;

public function list_map_2 "* function list_map_2
 ** Takes a list and a function and two extra arguments passed to the function.
 ** The function produces one new value which is used for creating a new list.
 ** For example,
 ** function if:(bool,'a,'a) => 'a
 ** list_map_2({true,false,false},1,0,if) => {1,0,0}
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  input Type_c in_type_c;
  output VType_dList out_vtype_dlist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    input Type_c in_type_c;
    output Type_d out_type_d;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
    replaceable type Type_d;
  end FuncType1;
  replaceable type Type_b;
  replaceable type Type_c;
  replaceable type Type_d;
  type VType_dList = list<Type_d>;
algorithm 
  out_vtype_dlist:=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b,in_type_c)
    local
      Type_d f_1;
      VType_dList r_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
      Type_b extraarg1;
      Type_c extraarg2;
    case ({},_,_,_) then {}; 
    case (f :: r,fn,extraarg1,extraarg2)
      equation 
        f_1 = fn(f, extraarg1, extraarg2);
        r_1 = list_map_2(r, fn, extraarg1, extraarg2); then f_1 :: r_1;
  end matchcontinue;
end list_map_2;

public function list_map_3 "* function list_map_3
 ** Takes a list and a function and three extra arguments passed to the function.
 ** The function produces one new value which is used for creating a new list.
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  input Type_c in_type_c;
  input Type_d in_type_d;
  output VType_eList out_vtype_elist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    input Type_c in_type_c;
    input Type_d in_type_d;
    output Type_e out_type_e;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
    replaceable type Type_d;
    replaceable type Type_e;
  end FuncType1;
  replaceable type Type_b;
  replaceable type Type_c;
  replaceable type Type_d;
  replaceable type Type_e;
  type VType_eList = list<Type_e>;
algorithm 
  out_vtype_elist:=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b,in_type_c,in_type_d)
    local
      Type_e f_1;
      VType_eList r_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
      Type_b extraarg1;
      Type_c extraarg2;
      Type_d extraarg3;
    case ({},_,_,_,_) then {}; 
    case (f :: r,fn,extraarg1,extraarg2,extraarg3)
      equation 
        f_1 = fn(f, extraarg1, extraarg2, extraarg3);
        r_1 = list_map_3(r, fn, extraarg1, extraarg2, extraarg3); then f_1 :: r_1;
  end matchcontinue;
end list_map_3;

public function list_map_3_2 "* function list_map_3_2
 ** Takes a list and a function and three extra arguments passed to the function.
 ** The function produces two values which is used for creating two new lists.
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  input Type_c in_type_c;
  input Type_d in_type_d;
  output VType_eList out_vtype_elist;
  output VType_fList out_vtype_flist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    input Type_c in_type_c;
    input Type_d in_type_d;
    output Type_e out_type_e;
    output Type_f out_type_f;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
    replaceable type Type_d;
    replaceable type Type_e;
    replaceable type Type_f;
  end FuncType1;
  replaceable type Type_b;
  replaceable type Type_c;
  replaceable type Type_d;
  replaceable type Type_e;
  type VType_eList = list<Type_e>;
  replaceable type Type_f;
  type VType_fList = list<Type_f>;
algorithm 
  (out_vtype_elist,out_vtype_flist):=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b,in_type_c,in_type_d)
    local
      Type_e f1_1;
      Type_f f2_1;
      VType_eList r1_1;
      VType_fList r2_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
      Type_b extraarg1;
      Type_c extraarg2;
      Type_d extraarg3;
    case ({},_,_,_,_) then ({},{}); 
    case (f :: r,fn,extraarg1,extraarg2,extraarg3)
      equation 
        (f1_1,f2_1) = fn(f, extraarg1, extraarg2, extraarg3);
        (r1_1,r2_1) = list_map_3_2(r, fn, extraarg1, extraarg2, extraarg3); then (f1_1 :: r1_1,f2_1 :: r2_1);
  end matchcontinue;
end list_map_3_2;

public function list_map_2_2 "* function: list_map_2_2
 ** Takes a list and a function with two extra arguments passed to the function.
 ** The function returns a tuple of two values which are used for creating two new lists
 ** For example,
 ** function foo(int,string,string) => (string,string) concatenates each string with 
 ** itself n times. foo(2,\"a\",b\") => (\"aa\",\"bb\")
 ** list_map_2_2 ({2,3},foo,\"a\",\"b\") => {(\"aa\",\"bb\"),(\"aa\",\"bbb\")}
*"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  input Type_c in_type_c;
  output TupleType_dType_eList out_tupletype_dtype_elist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    input Type_c in_type_c;
    output Type_d out_type_d;
    output Type_e out_type_e;
  protected 
    replaceable type Type_b;
    replaceable type Type_c;
    replaceable type Type_d;
    replaceable type Type_e;
  end FuncType1;
  replaceable type Type_b;
  replaceable type Type_c;
  replaceable type Type_d;
  replaceable type Type_e;
  type TupleType_dType_e = tuple<Type_d,Type_e>;
  type TupleType_dType_eList = list<TupleType_dType_e>;
algorithm 
  out_tupletype_dtype_elist:=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b,in_type_c)
    local
      Type_d f1;
      Type_e f2;
      TupleType_dType_eList r_1;
      Type_a f;
      VType_aList r;
      FuncType1 fn;
      Type_b extraarg1;
      Type_c extraarg2;
    case ({},_,_,_) then {}; 
    case (f :: r,fn,extraarg1,extraarg2)
      equation 
        (f1,f2) = fn(f, extraarg1, extraarg2);
        r_1 = list_map_2_2(r, fn, extraarg1, extraarg2); then (f1,f2) :: r_1;
  end matchcontinue;
end list_map_2_2;

public function list_map_0 "* function: list_map_0
 ** Takes a list and a function which does not return a value
 ** The function is probably a function with side effects, like print.
 ** For example,
 ** list_map_0({\"a\",\"b\",\"c\"},print) => ()
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
  end FuncType1;
algorithm 
  _:=
  matchcontinue (in_vtype_alist,in_functype1)
    local
      Type_a f;
      VType_aList r;
      FuncType1 fn;
    case ({},_) then (); 
    case (f :: r,fn)
      equation 
        fn(f);
        list_map_0(r, fn); then ();
  end matchcontinue;
end list_map_0;

public function list_list_map "* function: list_list_map 
 ** Takes a list of lists and a function producing one value.
 ** The function is applied to each element of the lists resulting
 ** in a new list of lists.
 ** For example,
 ** list_list_map({ {1,2},{3},{4}},int_string) => { {\"1\",\"2\"},{\"3\"},{\"4\"} }
 *"
  input VType_aListList in_vtype_alistlist;
  input FuncType1 in_functype1;
  output VType_bListList out_vtype_blistlist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  type VType_aListList = list<VType_aList>;
  function FuncType1
    input Type_a in_type_a;
    output Type_b out_type_b;
  protected 
    replaceable type Type_b;
  end FuncType1;
  replaceable type Type_b;
  type VType_bList = list<Type_b>;
  type VType_bListList = list<VType_bList>;
algorithm 
  out_vtype_blistlist:=
  matchcontinue (in_vtype_alistlist,in_functype1)
    local
      VType_bList f_1;
      VType_bListList r_1;
      VType_aList f;
      VType_aListList r;
      FuncType1 fn;
    case ({},_) then {}; 
    case (f :: r,fn)
      equation 
        f_1 = list_map(f, fn);
        r_1 = list_list_map(r, fn); then f_1 :: r_1;
  end matchcontinue;
end list_list_map;

public function list_fold "* function: list_fold
 ** Takes a list and a function operating on list elements having an extra argument that is 'updated'
 ** thus returned from the function. The third argument is the startvalue for the updated value.
 ** list_fold will call the function for each element in a sequence, updating the startvalue 
 ** For example,
 ** list_fold({1,2,3},int_add,2) =>  8
 ** int_add(1,2) => 3, int_add(2,3) => 5, int_add(3,5) => 8 
*"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  input Type_b in_type_b;
  output Type_b out_type_b;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    output Type_b out_type_b;
  protected 
    replaceable type Type_b;
  end FuncType1;
  replaceable type Type_b;
algorithm 
  out_type_b:=
  matchcontinue (in_vtype_alist,in_functype1,in_type_b)
    local
      FuncType1 r;
      Type_b b,b_1,b_2;
      Type_a l;
      VType_aList lst;
    case ({},r,b) then b; 
    case (l :: lst,r,b)
      equation 
        b_1 = r(l, b);
        b_2 = list_fold(lst, r, b_1); then b_2;
  end matchcontinue;
end list_fold;

public function list_list_reverse "* function: list_list_reverse
 ** Takes a list of lists and reverses it at both levels, i.e. both the list itself
 ** and each sublist
 ** For example,
 ** list_list_reverse({{1,2},{3,4,5},{6} }) => { {6}, {5,4,3}, {2,1} }
 *"
  input VType_aListList lsts;
  output VType_aListList lsts_2;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  type VType_aListList = list<VType_aList>;
  VType_aListList lsts_1,lsts_2;
algorithm 
  lsts_1 := list_map(lsts, list_reverse);
  lsts_2 := list_reverse(lsts_1);
end list_list_reverse;

public function list_thread "* function: list_thread
 ** Takes two lists of the same type and threads them togheter.
 ** For eample,
 ** list_thread({1,2,3},{4,5,6}) => {4,1,5,2,6,3}
 *"
  input VType_aList in_vtype_alist1;
  input VType_aList in_vtype_alist2;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_alist1,in_vtype_alist2)
    local
      VType_aList r_1,c,d,ra,rb;
      Type_a fa,fb;
    case ({},{}) then {}; 
    case (fa :: ra,fb :: rb)
      equation 
        r_1 = list_thread(ra, rb);
        c = fb :: r_1;
        d = fa :: c; then d;
  end matchcontinue;
end list_thread;

public function list_thread_map "* function: list_thread_map
 ** Takes two lists and a function and threads and maps the elements of the two lists
 ** creating a new list.
 ** For example,
 ** list_thread_map({1,2},{3,4},int_add) => {1+3, 2+4}
 *"
  input VType_aList in_vtype_alist;
  input VType_bList in_vtype_blist;
  input FuncType1 in_functype1;
  output VType_cList out_vtype_clist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  replaceable type Type_b;
  type VType_bList = list<Type_b>;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    output Type_c out_type_c;
  protected 
    replaceable type Type_c;
  end FuncType1;
  replaceable type Type_c;
  type VType_cList = list<Type_c>;
algorithm 
  out_vtype_clist:=
  matchcontinue (in_vtype_alist,in_vtype_blist,in_functype1)
    local
      Type_c fr;
      VType_cList res;
      Type_a fa;
      VType_aList ra;
      Type_b fb;
      VType_bList rb;
      FuncType1 fn;
    case ({},{},_) then {}; 
    case (fa :: ra,fb :: rb,fn)
      equation 
        fr = fn(fa, fb);
        res = list_thread_map(ra, rb, fn); then fr :: res;
  end matchcontinue;
end list_thread_map;

public function list_thread_tuple "* function: list_thread_tuple
 ** Takes two lists and threads the arguments into a list of tuples
 ** consisting of the two element types.
 ** For example,
 ** list_thread_tuple({1,2,3},{true,false,true}) => {(1,true),(2,false),(3,true)}
 *"
  input VType_aList in_vtype_alist;
  input VType_bList in_vtype_blist;
  output TupleType_aType_bList out_tupletype_atype_blist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  replaceable type Type_b;
  type VType_bList = list<Type_b>;
  type TupleType_aType_b = tuple<Type_a,Type_b>;
  type TupleType_aType_bList = list<TupleType_aType_b>;
algorithm 
  out_tupletype_atype_blist:=
  matchcontinue (in_vtype_alist,in_vtype_blist)
    local
      TupleType_aType_bList r;
      Type_a fa;
      VType_aList ra;
      Type_b fb;
      VType_bList rb;
    case ({},{}) then {}; 
    case (fa :: ra,fb :: rb)
      equation 
        r = list_thread_tuple(ra, rb); then (fa,fb) :: r;
  end matchcontinue;
end list_thread_tuple;

public function list_list_thread_tuple
  input VType_aListList in_vtype_alistlist;
  input VType_bListList in_vtype_blistlist;
  output TupleType_aType_bListList out_tupletype_atype_blistlist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  type VType_aListList = list<VType_aList>;
  replaceable type Type_b;
  type VType_bList = list<Type_b>;
  type VType_bListList = list<VType_bList>;
  type TupleType_aType_b = tuple<Type_a,Type_b>;
  type TupleType_aType_bList = list<TupleType_aType_b>;
  type TupleType_aType_bListList = list<TupleType_aType_bList>;
algorithm 
  out_tupletype_atype_blistlist:=
  matchcontinue (in_vtype_alistlist,in_vtype_blistlist)
    local
      TupleType_aType_bList f;
      TupleType_aType_bListList r;
      VType_aList fa;
      VType_aListList ra;
      VType_bList fb;
      VType_bListList rb;
    case ({},{}) then {}; 
    case (fa :: ra,fb :: rb)
      equation 
        f = list_thread_tuple(fa, fb);
        r = list_list_thread_tuple(ra, rb); then f :: r;
  end matchcontinue;
end list_list_thread_tuple;

public function list_select "* function: list_select
 ** This function retrieves all elements of a list for which
 ** the passed function evaluates to true. The elements that evaluates to false 
 ** are thus removed from the list.
*"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_alist,in_functype1)
    local
      VType_aList xs_1,xs;
      Type_a x;
      FuncType1 cond;
    case ({},_) then {}; 
    case (x :: xs,cond)
      equation 
        true = cond(x);
        xs_1 = list_select(xs, cond); then x :: xs_1;
    case (x :: xs,cond)
      equation 
        false = cond(x);
        xs_1 = list_select(xs, cond); then xs_1;
  end matchcontinue;
end list_select;

public function list_select_1 "* function list_select_1
 ** Same as list_select above, but with extra argument to testing function.
 *"
  input VType_aList in_vtype_alist;
  input Type_b in_type_b;
  input FuncType1 in_functype1;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  replaceable type Type_b;
  function FuncType1
    input Type_a in_type_a;
    input Type_b in_type_b;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_alist,in_type_b,in_functype1)
    local
      Type_b arg;
      VType_aList xs_1,xs;
      Type_a x;
      FuncType1 cond;
    case ({},arg,_) then {}; 
    case (x :: xs,arg,cond)
      equation 
        true = cond(x, arg);
        xs_1 = list_select_1(xs, arg, cond); then x :: xs_1;
    case (x :: xs,arg,cond)
      equation 
        false = cond(x, arg);
        xs_1 = list_select_1(xs, arg, cond); then xs_1;
  end matchcontinue;
end list_select_1;

public function list_position "* function: list_position
 ** Takes a value and a list of values and returns the (first) position
 ** the value has in the list. Position index start at zero, such that list_nth can
 ** be used on the resulting position directly.
 ** For example,
 ** list_position(2,{0,1,2,3}) => 2
 *"
  input Type_a x;
  input VType_aList ys;
  output Integer n;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  n := list_pos(x, ys, 0);
end list_position;

protected function list_pos "* helper function to list_position *"
  input Type_a in_type_a;
  input VType_aList in_vtype_alist;
  input Integer in_integer;
  output Integer out_integer;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  out_integer:=
  matchcontinue (in_type_a,in_vtype_alist,in_integer)
    local
      Type_a x,y,i;
      VType_aaList ys;
      Integer i_1,n;
    case (x,y :: ys,i)
      equation 
        equality(x = y); then i;
    case (x,y :: ys,i)
      local Integer i;
      equation 
        failure(equality(x = y));
        i_1 = i + 1;
        n = list_pos(x, ys, i_1); then n;
  end matchcontinue;
end list_pos;

public function list_getmember "* function: list_getmember
 ** Takes a value and a list of values and returns the value 
 ** if present in the list. If not present, the function will fail.
 ** For example,
 ** list_getmember(0,{1,2,3}) => fail
 ** list_getmember(1,{1,2,3}) => 1
 *"
  input Type_1_a in_type_1_a;
  input VType_1_aList in_vtype_1_alist;
  output Type_1_a out_type_1_a;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
algorithm 
  out_type_1_a:=
  matchcontinue (in_type_1_a,in_vtype_1_alist)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      Type_a x,y,res;
      VType_aList ys;
    case (_,{}) then fail(); 
    case (x,y :: ys)
      equation 
        equality(x = y); then y;
    case (x,y :: ys)
      equation 
        failure(equality(x = y));
        res = list_getmember(x, ys); then res;
  end matchcontinue;
end list_getmember;

public function list_deletemember "* function: list_deletemember
 ** Takes a list and a value and deletes the first occurence of the value in the list
 ** For example,
 ** list_deletemember({1,2,3,2},2) => {1,3,2}
 *"
  input VType_1_aList in_vtype_1_alist;
  input Type_1_a in_type_1_a;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alist,in_type_1_a)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      Integer pos;
      VType_aList lst_1,lst;
      Type_a elt;
    case (lst,elt)
      equation 
        pos = list_position(elt, lst);
        lst_1 = list_delete(lst, pos); then lst_1;
    case (lst,_) then lst; 
  end matchcontinue;
end list_deletemember;

protected function list_deletemember_p "* function: list_deletemember_p
 ** Takes a list and a value and a comparison function and deletes the first occurence of 
 ** the value in the list
 ** For example,
 ** list_deletemember({1,2,3,2},2,int_eq) => {1,3,2}
 *"
  input VType_1_aList in_vtype_1_alist;
  input Type_1_a in_type_1_a;
  input FuncType1 in_functype1;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alist,in_type_1_a,in_functype1)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      end FuncType1;
      Type_a elt_1,elt;
      Integer pos;
      VType_aList lst_1,lst;
      FuncType1 cond;
    case (lst,elt,cond)
      equation 
        elt_1 = list_getmember_p(elt, lst, cond) " A bit ugly ";
        pos = list_position(elt_1, lst);
        lst_1 = list_delete(lst, pos); then lst_1;
    case (lst,_,_) then lst; 
  end matchcontinue;
end list_deletemember_p;

public function list_getmember_p "* function list_getmember_p
 ** Takes a value and a list of values and a comparison function over two values.
 ** If the value is present in the list (using the comparison function returning true)
 ** the value is returned, otherwise the function fails.
 ** For example,
 ** function equal_lenght(string,string) returns true if the strings are of same length
 ** list_getmember_p(\"a\",{\"bb\",\"b\",\"ccc\"},equal_length) => \"b\"
 *"
  input Type_1_a in_type_1_a;
  input VType_1_aList in_vtype_1_alist;
  input FuncType1 in_functype1;
  output Type_1_a out_type_1_a;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_type_1_a:=
  matchcontinue (in_type_1_a,in_vtype_1_alist,in_functype1)
    local
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      protected 
        replaceable type Type_a;
      end FuncType1;
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      FuncType1 p;
      Type_a x,y,res;
      VType_aList ys;
    case (_,{},p) then fail(); 
    case (x,y :: ys,p)
      equation 
        true = p(x, y); then y;
    case (x,y :: ys,p)
      equation 
        false = p(x, y);
        res = list_getmember_p(x, ys, p); then res;
  end matchcontinue;
end list_getmember_p;

public function list_union_elt "* function: list_union_elt
 ** Takes a value and a list of values and inserts the value into the list if 
 ** it is not already in the list.
 ** If it is in the list it is not inserted.
 ** For example,
 ** list_union_elt(1,{2,3}) => {1,2,3}
 ** list_union_elt(0,{0,1,2}) => {0,1,2}
*"
  input Type_1_a in_type_1_a;
  input VType_1_aList in_vtype_1_alist;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_type_1_a,in_vtype_1_alist)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      Type_a x;
      VType_aList lst;
    case (x,lst)
      equation 
        _ = list_getmember(x, lst); then lst;
    case (x,lst)
      equation 
        failure(_ = list_getmember(x, lst)); then x :: lst;
  end matchcontinue;
end list_union_elt;

public function list_union "* function list_union
 ** Takes two lists and returns the union of the two lists, i.e. a list of all elements combined
 ** without duplicates.
 ** For example,
 ** list_union({0,1},{2,1}) => {0,1,2}
*"
  input VType_1_aList in_vtype_1_alist1;
  input VType_1_aList in_vtype_1_alist2;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alist1,in_vtype_1_alist2)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      VType_aList res,r1,xs,lst2;
      Type_a x;
    case ({},res) then res; 
    case (x :: xs,lst2)
      equation 
        r1 = list_union_elt(x, lst2);
        res = list_union(xs, r1); then res;
  end matchcontinue;
end list_union;

public function list_list_union "* function: list_list_union
 ** Takes a list of lists and returns the union of the sublists
 ** For example,
 ** list_list_union({{1},{1,2},{3,4},{5}}) => {1,2,3,4,5}
 *"
  input VType_1_aListList in_vtype_1_alistlist;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  type VType_1_aListList = list<VType_1_aList>;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alistlist)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      type VType_aListList = list<VType_aList>;
      VType_aList x,r1,res,x1,x2;
      VType_aListList rest;
    case ({}) then {}; 
    case ({x}) then x; 
    case (x1 :: x2 :: rest)
      equation 
        r1 = list_union(x1, x2);
        res = list_list_union(r1 :: rest); then res;
  end matchcontinue;
end list_list_union;

public function list_union_elt_p "* function: list_union_elt_p
 ** Takes an elemement and a list and a comparison function over the two values.
 ** It returns the list with the element inserted if not already present in the
 ** list, according to the comparison function.
 ** For example,
 ** list_union_elt_p(1,{2,3},int_eq) => {1,2,3}
 *"
  input Type_1_a in_type_1_a;
  input VType_1_aList in_vtype_1_alist;
  input FuncType1 in_functype1;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_type_1_a,in_vtype_1_alist,in_functype1)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      end FuncType1;
      Type_a x;
      VType_aList lst;
      FuncType1 p;
    case (x,lst,p)
      equation 
        _ = list_getmember_p(x, lst, p); then lst;
    case (x,lst,p)
      equation 
        failure(_ = list_getmember_p(x, lst, p)); then x :: lst;
  end matchcontinue;
end list_union_elt_p;

public function list_union_p "* function: list_union_p
 ** Takes two lists and a comparison function over two elements of the list.
 ** It returns the union of the two lists, using the comparison function passed as argument
 ** to determine identity between two elements.
 ** For example
 ** given the function equal_lenght(string,string) returning true if the strings are of same length
 ** list_union_p({\"a\",\"aa\"},{\"b\",\"bbb\"},equal_length) => {\"a\",\"aa\",\"bbb\"}
 *"
  input VType_1_aList in_vtype_1_alist1;
  input VType_1_aList in_vtype_1_alist2;
  input FuncType1 in_functype13;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alist1,in_vtype_1_alist2,in_functype13)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      end FuncType1;
      VType_aList res,r1,xs,lst2;
      FuncType1 p;
      Type_a x;
    case ({},res,p) then res; 
    case (x :: xs,lst2,p)
      equation 
        r1 = list_union_elt_p(x, lst2, p);
        res = list_union_p(xs, r1, p); then res;
  end matchcontinue;
end list_union_p;

public function list_intersection_p "* function: list_intersection_p
 ** Takes two lists and a comparison function over two elements of the list.
 ** It returns the intersection of the two lists, using the comparison function passed as 
 ** argument to determine identity between two elements.
 ** For example
 ** given the function string_equal(string,string) returning true if the strings are equal
 ** list_intersection_p({\"a\",\"aa\"},{\"b\",\"aa\"},string_equal) => {\"aa\"}
 *"
  input VType_1_aList in_vtype_1_alist1;
  input VType_1_aList in_vtype_1_alist2;
  input FuncType1 in_functype13;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alist1,in_vtype_1_alist2,in_functype13)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      end FuncType1;
      VType_aList res,xs1,xs2;
      Type_a x1;
      FuncType1 cond;
    case ({},_,_) then {}; 
    case (x1 :: xs1,xs2,cond)
      equation 
        _ = list_getmember_p(x1, xs2, cond);
        res = list_intersection_p(xs1, xs2, cond); then x1 :: res;
    case (x1 :: xs1,xs2,cond)
      equation 
        res = list_intersection_p(xs1, xs2, cond) " not list_getmember_p(x1,xs2,cond) => _ "; then res;
  end matchcontinue;
end list_intersection_p;

public function list_setdifference_p
  input VType_1_aList in_vtype_1_alist1;
  input VType_1_aList in_vtype_1_alist2;
  input FuncType1 in_functype13;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alist1,in_vtype_1_alist2,in_functype13)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      end FuncType1;
      VType_aList a,a_1,a_2,xs;
      FuncType1 cond;
      Type_a x1;
    case (a,{},cond) then a; 
    case (a,x1 :: xs,cond)
      equation 
        a_1 = list_deletemember_p(a, x1, cond);
        a_2 = list_setdifference_p(a_1, xs, cond); then a_2;
    case (_,_,_)
      equation 
        print("-list_setdifference_p failed\n"); then fail();
  end matchcontinue;
end list_setdifference_p;

public function list_list_union_p "* function: list_list_union_p
 ** Takes a list of lists and a comparison function over two elements of the lists.
 ** It returns the union of all sublists using the comparison function for identity.
 ** For example,
 ** list_list_union_p({{1},{1,2},{3,4}},int_eq) => {1,2,3,4}
 *"
  input VType_1_aListList in_vtype_1_alistlist;
  input FuncType1 in_functype1;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
  type VType_1_aListList = list<VType_1_aList>;
  function FuncType1
    input Type_1_a in_type_1_a1;
    input Type_1_a in_type_1_a2;
    output Boolean out_boolean;
  end FuncType1;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_vtype_1_alistlist,in_functype1)
    local
      function FuncType1
        input Type_a in_type_a1;
        input Type_a in_type_a2;
        output Boolean out_boolean;
      protected 
        replaceable type Type_a;
      end FuncType1;
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      type VType_aListList = list<VType_aList>;
      FuncType1 p;
      VType_aList x,r1,res,x1,x2;
      VType_aListList rest;
    case ({},p) then {}; 
    case ({x},p) then x; 
    case (x1 :: x2 :: rest,p)
      equation 
        r1 = list_union_p(x1, x2, p);
        res = list_list_union_p(r1 :: rest, p); then res;
  end matchcontinue;
end list_list_union_p;

public function list_replaceat "* function: list_replaceat
 ** Takes an element, a position and a list and replaces the value at the given position in 
 ** the list. Position is an integer between 0 and n-1 for a list of n elements
 ** For example,
 ** list_replaceat(\"A\", 2, {\"a\",\"b\",\"c\"}) => {\"a\",\"b\",\"A\"}
 *"
  input Type_1_a in_type_1_a;
  input Integer in_integer;
  input VType_1_aList in_vtype_1_alist;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_type_1_a,in_integer,in_vtype_1_alist)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      Type_a x,y;
      VType_aList ys,res;
      Integer nn,n;
    case (x,0,y :: ys) then x :: ys; 
    case (x,n,y :: ys) " rule	print \"-list_replaceat failed\n\" 
	-----------------------
	list_replaceat(_,_,_) => fail"
      equation 
        (n >= 1) = true;
        nn = n - 1;
        res = list_replaceat(x, nn, ys); then y :: res;
  end matchcontinue;
end list_replaceat;

public function list_replaceat_with_fill "* function: list_replaceat_with_fill
 ** Takes 
 ** - an element, 
 ** - a position 
 ** - a list and 
 ** - a fill value 
 ** The function replaces the value at the given position in the list, if the given position is 
 ** out of range, the fill value is used to padd the list up to that element position and then
 ** insert the value at the position
 ** 
 ** For example,
 ** list_replaceat_withfill(\"A\", 5, {\"a\",\"b\",\"c\"},\"dummy\") => {\"a\",\"b\",\"c\",\"dummy\",\"A\"}
 *"
  input Type_1_a in_type_1_a1;
  input Integer in_integer2;
  input VType_1_aList in_vtype_1_alist3;
  input Type_1_a in_type_1_a4;
  output VType_1_aList out_vtype_1_alist;
protected 
  replaceable type Type_1_a;
  type VType_1_aList = list<Type_1_a>;
algorithm 
  out_vtype_1_alist:=
  matchcontinue (in_type_1_a1,in_integer2,in_vtype_1_alist3,in_type_1_a4)
    local
      replaceable type Type_a;
      type VType_aList = list<Type_a>;
      Type_a x,fillv,y;
      VType_aList ys,res,res_1;
      Integer numfills_1,numfills,nn,n,p;
      String pos;
    case (x,0,{},fillv) then {x}; 
    case (x,0,y :: ys,fillv) then x :: ys; 
    case (x,1,{},fillv) then {fillv,x}; 
    case (x,numfills,{},fillv)
      equation 
        (numfills > 1) = true;
        numfills_1 = numfills - 1;
        res = list_fill(fillv, numfills_1);
        res_1 = list_append(res, {x}); then res_1;
    case (x,n,y :: ys,fillv)
      equation 
        (n >= 1) = true;
        nn = n - 1;
        res = list_replaceat_with_fill(x, nn, ys, fillv); then y :: res;
    case (_,p,_,_)
      equation 
        print("-list_replaceat_with_fill failed row: ");
        pos = int_string(p);
        print(pos);
        print("\n"); then fail();
  end matchcontinue;
end list_replaceat_with_fill;

public function list_reduce "* function: list_reduce
 ** Takes a list and a function operating on two elements of the list.
 ** The function performs a reduction of the lists to a single value using the function.
 ** For example,
 ** list_reduce({1,2,3},int_add) => 6
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  output Type_a out_type_a;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a1;
    input Type_a in_type_a2;
    output Type_a out_type_a;
  end FuncType1;
algorithm 
  out_type_a:=
  matchcontinue (in_vtype_alist,in_functype1)
    local
      Type_a e,res,a,b,res1,res2;
      FuncType1 r;
      VType_aList xs;
    case ({e},r) then e; 
    case ({a,b},r)
      equation 
        res = r(a, b); then res;
    case (a :: b :: (xs = _ :: _),r)
      equation 
        res1 = r(a, b);
        res2 = list_reduce(xs, r);
        res = r(res1, res2); then res;
  end matchcontinue;
end list_reduce;

public function array_replaceat_with_fill "* function: array_replaceat_with_fill
 ** Takes 
 ** - an element, 
 ** - a position 
 ** - an array and 
 ** - a fill value 
 ** The function replaces the value at the given position in the array, if the given position is 
 ** out of range, the fill value is used to padd the array up to that element position and then
 ** insert the value at the position
 ** 
 ** For example,
 ** array_replaceat_withfill(\"A\", 5, {\"a\",\"b\",\"c\"},\"dummy\") => {\"a\",\"b\",\"c\",\"dummy\",\"A\"}
 *"
  input Type_1_a in_type_1_a1;
  input Integer in_integer2;
  input VType_1_aArray in_vtype_1_aarray3;
  input Type_1_a in_type_1_a4;
  output VType_1_aArray out_vtype_1_aarray;
protected 
  replaceable type Type_1_a;
  type VType_1_aArray = Type_1_a[:];
algorithm 
  out_vtype_1_aarray:=
  matchcontinue (in_type_1_a1,in_integer2,in_vtype_1_aarray3,in_type_1_a4)
    local
      replaceable type Type_a;
      type VType_aArray = Type_a[:];
      Integer alen,pos,pos_1;
      VType_aArray res,arr,newarr,res_1;
      Type_a x,fillv;
    case (x,pos,arr,fillv)
      equation 
        alen = array_length(arr) " Replacing element with index in range of the array";
        (pos < alen) = true;
        res = array_update(arr, pos + 1, x); then res;
    case (x,pos,arr,fillv)
      equation 
        pos_1 = pos + 1 " Replacing element out of range of array, create new array, and copy elts.";
        newarr = fill(fillv, pos_1);
        res = array_copy(arr, newarr);
        res_1 = array_update(res, pos + 1, x); then res_1;
    case (_,_,_,_)
      equation 
        print("-array_replaceat_with_fill failed\n"); then fail();
  end matchcontinue;
end array_replaceat_with_fill;

public function array_expand "* function: array_expand
 ** Increases the number of elements of a list with n.
 ** Each of the new elements have the value v.
 *"
  input Integer n;
  input VType_1_aArray arr;
  input Type_1_a v;
  output VType_1_aArray newarr_1;
protected 
  replaceable type Type_1_a;
  type VType_1_aArray = Type_1_a[:];
  replaceable type Type_a;
  type VType_aArray = Type_a[:];
  Integer len,newlen;
  VType_aArray newarr,newarr_1;
algorithm 
  len := array_length(arr);
  newlen := n + len;
  newarr := fill(v, newlen);
  newarr_1 := array_copy(arr, newarr);
end array_expand;

public function array_n_copy
  input VType_1_aArray src;
  input VType_1_aArray dst;
  input Integer n;
  output VType_1_aArray dst_1;
protected 
  replaceable type Type_1_a;
  type VType_1_aArray = Type_1_a[:];
  replaceable type Type_a;
  type VType_aArray = Type_a[:];
  Integer n_1;
  VType_aArray dst_1;
algorithm 
  n_1 := n - 1;
  dst_1 := array_copy2(src, dst, n_1);
end array_n_copy;

public function array_copy "* function: array_copy
 ** copies all values in src array into dest array.
 ** The function fails if all elements can not be fit into dest array.
 *"
  input VType_1_aArray in_vtype_1_aarray1;
  input VType_1_aArray in_vtype_1_aarray2;
  output VType_1_aArray out_vtype_1_aarray;
protected 
  replaceable type Type_1_a;
  type VType_1_aArray = Type_1_a[:];
algorithm 
  out_vtype_1_aarray:=
  matchcontinue (in_vtype_1_aarray1,in_vtype_1_aarray2)
    local
      replaceable type Type_a;
      type VType_aArray = Type_a[:];
      Integer srclen,dstlen;
      VType_aArray src,dst,dst_1;
    case (src,dst) "src dst"
      equation 
        srclen = array_length(src);
        dstlen = array_length(dst);
        (srclen > dstlen) = true;
        print(
          "-array_copy failed. Can not fit elements into dest array\n"); then fail();
    case (src,dst)
      equation 
        srclen = array_length(src);
        srclen = srclen - 1;
        dst_1 = array_copy2(src, dst, srclen); then dst_1;
  end matchcontinue;
end array_copy;

protected function array_copy2 "current pos"
  input VType_1_aArray in_vtype_1_aarray1;
  input VType_1_aArray in_vtype_1_aarray2;
  input Integer in_integer3;
  output VType_1_aArray out_vtype_1_aarray;
protected 
  replaceable type Type_1_a;
  type VType_1_aArray = Type_1_a[:];
algorithm 
  out_vtype_1_aarray:=
  matchcontinue (in_vtype_1_aarray1,in_vtype_1_aarray2,in_integer3)
	local
      replaceable type Type_a;
      type VType_aArray = Type_a[:];
      VType_aArray src,dst,dst_1,dst_2;
      Type_a elt;
      Integer pos;
    case (src,dst,-1) then dst; 
    case (src,dst,pos)
      equation 
        elt = src[pos + 1]; 
        dst_1 = array_update(dst, pos + 1, elt); 
        pos = pos - 1;
        dst_2 = array_copy2(src, dst_1, pos); 
        then dst_2;
  end matchcontinue;
end array_copy2;

public function tuple2_1 "* function: tuple2_1
 ** Takes a tuple of two values and returns the first value.
 ** For example,
 ** tuple2_1((\"a\",1)) => \"a\"
 *"
  input TupleType_aType_b in_tupletype_atype_b;
  output Type_a out_type_a;
protected 
  replaceable type Type_a;
  replaceable type Type_b;
  type TupleType_aType_b = tuple<Type_a,Type_b>;
algorithm 
  out_type_a:=
  matchcontinue (in_tupletype_atype_b)
    local Type_a a;
    case ((a,_)) then a; 
  end matchcontinue;
end tuple2_1;

public function tuple2_2 "* function: tuple2_2
 ** Takes a tuple of two values and returns the second value.
 ** For example,
 ** tuple2_2((\"a\",1)) => 1
 *"
  input TupleType_aType_b in_tupletype_atype_b;
  output Type_b out_type_b;
protected 
  replaceable type Type_a;
  replaceable type Type_b;
  type TupleType_aType_b = tuple<Type_a,Type_b>;
algorithm 
  out_type_b:=
  matchcontinue (in_tupletype_atype_b)
    local Type_b b;
    case ((_,b)) then b; 
  end matchcontinue;
end tuple2_2;

public function split_tuple2_list "* function: split_tuple2_list
 ** Takes a list of two-tuples and splits it into two lists.
 ** For example,
 ** split_tuple2_list({(\"a\",1),(\"b\",2),(\"c\",3)}) => ({\"a\",\"b\",\"c\"}, {1,2,3})
 *"
  input TupleType_aType_bList in_tupletype_atype_blist;
  output VType_aList out_vtype_alist;
  output VType_bList out_vtype_blist;
protected 
  replaceable type Type_a;
  replaceable type Type_b;
  type TupleType_aType_b = tuple<Type_a,Type_b>;
  type TupleType_aType_bList = list<TupleType_aType_b>;
  type VType_aList = list<Type_a>;
  type VType_bList = list<Type_b>;
algorithm 
  (out_vtype_alist,out_vtype_blist):=
  matchcontinue (in_tupletype_atype_blist)
    local
      VType_aList xs;
      VType_bList ys;
      Type_a x;
      Type_b y;
      TupleType_aType_bList rest;
    case ({}) then ({},{}); 
    case ((x,y) :: rest)
      equation 
        (xs,ys) = split_tuple2_list(rest); then (x :: xs,y :: ys);
  end matchcontinue;
end split_tuple2_list;

public function if_ "* function: if
 ** Takes a boolean and two values.
 ** Returns the first value (second argument) if the boolean value is true, otherwise 
 ** the second value (third argument) is returned.
 ** For example,
 ** if(true,\"a\",\"b\") => \"a\"
 *"
  input Boolean in_boolean1;
  input Type_a in_type_a2;
  input Type_a in_type_a3;
  output Type_a out_type_a;
protected 
  replaceable type Type_a;
algorithm 
  out_type_a:=
  matchcontinue (in_boolean1,in_type_a2,in_type_a3)
    local Type_a r;
    case (true,r,_) then r; 
    case (false,_,r) then r; 
  end matchcontinue;
end if_;

public function string_append_list "* function string_append_list
 ** Takes a list of strings and appends them.
 ** For example,
 ** string_append_list({\"foo\", \" \", \"bar\"}) => \"foo bar\"
 *"
  input StringList in_stringlist;
  output String out_string;
protected 
  type StringList = list<String>;
algorithm 
  out_string:=
  matchcontinue (in_stringlist)
    local
      String f,r_1,str;
      StringList r;
    case {} then ""; 
    case {f} then f; 
    case f :: r
      equation 
        r_1 = string_append_list(r);
        str = string_append(f, r_1); then str;
  end matchcontinue;
end string_append_list;

public function string_delimit_list "* function string_delimit_list
 ** Takes a list of strings and a string delimiter and appends all list elements with
 ** the string delimiter inserted between elements.
 ** For example,
 ** string_delimit_list({\"x\",\"y\",\"z\"}, \", \") => \"x, y, z\"
 *"
  input StringList in_stringlist;
  input String in_string;
  output String out_string;
protected 
  type StringList = list<String>;
algorithm 
  out_string:=
  matchcontinue (in_stringlist,in_string)
    local
      String f,delim,str1,str2,str;
      StringList r;
    case ({},_) then ""; 
    case ({f},delim) then f; 
    case (f :: r,delim)
      equation 
        str1 = string_delimit_list(r, delim);
        str2 = string_append(f, delim);
        str = string_append(str2, str1); then str;
  end matchcontinue;
end string_delimit_list;

public function string_delimit_list_2sep "* function: string_delimit_list_2sep
 ** author: PA
 **
 ** This function is similar to string_delimit_list, i.e it inserts string delimiters between 
 ** consecutive strings in a list. But it also count the lists and inserts a second string delimiter
 ** when the counter is reached. This can be used when for instance outputting large lists of values
 ** and a newline is needed after ten or so items.
*"
  input StringList str;
  input String sep1;
  input String sep2;
  input Integer n;
  output String res;
protected 
  type StringList = list<String>;
algorithm 
  res := string_delimit_list_2sep2(str, sep1, sep2, n, 0);
end string_delimit_list_2sep;

protected function string_delimit_list_2sep2
  input StringList in_stringlist1;
  input String in_string2;
  input String in_string3;
  input Integer in_integer4;
  input Integer in_integer5;
  output String out_string;
protected 
  type StringList = list<String>;
algorithm 
  out_string:=
  matchcontinue (in_stringlist1,in_string2,in_string3,in_integer4,in_integer5)
    local
      String s,str1,str,f,sep1,sep2;
      StringList r;
      Integer n,iter_1,iter;
    case ({},_,_,_,_) then ""; 
    case ({s},_,_,_,_) then s; 
    case (f :: r,sep1,sep2,n,0)
      equation 
        str1 = string_delimit_list_2sep2(r, sep1, sep2, n, 1) " special case for first element";
        str = string_append_list({f,sep1,str1}); then str;
    case (f :: r,sep1,sep2,n,iter)
      equation 
        0 = int_mod(iter, n) " insert second delimiter";
        iter_1 = iter + 1;
        str1 = string_delimit_list_2sep2(r, sep1, sep2, n, iter_1);
        str = string_append_list({f,sep1,sep2,str1}); then str;
    case (f :: r,sep1,sep2,n,iter)
      equation 
        iter_1 = iter + 1 " not inserting second delimiter";
        str1 = string_delimit_list_2sep2(r, sep1, sep2, n, iter_1);
        str = string_append_list({f,sep1,str1}); then str;
    case (_,_,_,_,_)
      equation 
        print("string_delimit_list_2sep2 failed\n"); then fail();
  end matchcontinue;
end string_delimit_list_2sep2;

public function string_delimit_list_no_empty "* function string_delimit_list_no_empty
 ** Takes a list of strings and a string delimiter and appends all list elements with
 ** the string delimiter inserted between elements.
 ** For example,
 ** string_delimit_list({\"x\",\"y\",\"z\"}, \", \") => \"x, y, z\"
 *"
  input StringList lst;
  input String delim;
  output String str;
protected 
  type StringList = list<String>;
  StringList lst1;
algorithm 
  lst1 := list_select(lst, is_not_empty_string);
  str := string_delimit_list(lst1, delim);
end string_delimit_list_no_empty;

public function string_replace_char "* function string_replace_char
 ** Takes a string and two chars and replaces the first char to 
 ** second char:
 ** example: string_replace_char(\"hej.b.c\",#\".\",#\"_\") => \"hej_b_c\"
 *"
  input String in_string1;
  input char in_char2;
  input char in_char3;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_string1,in_char2,in_char3)
    local
      type charList = list<char>;
      charList strList,resList;
      String res,str;
      char fromChar,toChar;
    case (str,fromChar,toChar)
      equation 
        strList = string_list(str);
        resList = string_replace_char2(strList, fromChar, toChar);
        res = list_string(resList); then res;
    case (strList,_,_)
      local String strList;
      equation 
        print("string_replace_char failed\n"); then strList;
  end matchcontinue;
end string_replace_char;

protected function string_replace_char2
  input charList in_charlist1;
  input char in_char2;
  input char in_char3;
  output charList out_charlist;
protected 
  type charList = list<char>;
algorithm 
  out_charlist:=
  matchcontinue (in_charlist1,in_char2,in_char3)
    local
      charList res,rest,strList;
      char firstChar,fromChar,toChar;
    case ({},_,_) then {}; 
    case (firstChar :: rest,fromChar,toChar)
      equation 
        equality(firstChar = fromChar);
        res = string_replace_char2(rest, fromChar, toChar); then toChar :: res;
    case (firstChar :: rest,fromChar,toChar)
      equation 
        failure(equality(firstChar = fromChar));
        res = string_replace_char2(rest, fromChar, toChar); then firstChar :: res;
    case (strList,_,_)
      equation 
        print("string_replace_char2 failed\n"); then strList;
  end matchcontinue;
end string_replace_char2;

public function string_split_at_char "* function string_split_at_char
 ** Takes a string and a char and split the string at the char
 ** example: string_split_at_char(\"hej.b.c\",#\".\") => {\"hej,\"b\",\"c\"}
 *"
  input String in_string;
  input char in_char;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_string,in_char)
    local
      type charList = list<char>;
      charList chrList;
      StringList stringList;
      String str,strList;
      char chr;
    case (str,chr)
      equation 
        chrList = string_list(str);
        stringList = string_split_at_char2(chrList, chr, {}) "list_string(resList) => res"; then stringList;
    case (strList,_)
      equation 
        print("string_split_at_char failed\n"); then {strList};
  end matchcontinue;
end string_split_at_char;

protected function string_split_at_char2
  input charList in_charlist1;
  input char in_char2;
  input charList in_charlist3;
  output StringList out_stringlist;
protected 
  type charList = list<char>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_charlist1,in_char2,in_charlist3)
    local
      charList chr_rest_1,chr_rest,chrList,rest,strList;
      String res;
      StringList res_str;
      char firstChar,chr;
    case ({},_,chr_rest)
      equation 
        chr_rest_1 = list_reverse(chr_rest);
        res = list_string(chr_rest_1); then {res};
    case (firstChar :: rest,chr,chr_rest)
      equation 
        equality(firstChar = chr);
        chrList = list_reverse(chr_rest) " this is needed because it returns the reversed list ";
        res = list_string(chrList);
        res_str = string_split_at_char2(rest, chr, {}); then res :: res_str;
    case (firstChar :: rest,chr,chr_rest)
      local StringList res;
      equation 
        failure(equality(firstChar = chr));
        res = string_split_at_char2(rest, chr, firstChar :: chr_rest); then res;
    case (strList,_,_)
      equation 
        print("string_split_at_char2 failed\n"); then fail();
  end matchcontinue;
end string_split_at_char2;

public function bool_or_list "* function bool_or_list
 ** Takes a list of boolean values and applies the boolean 'or' operator  to the list elements
 ** For example
 ** bool_or_list({true,false,false}) => true
 ** bool_or_list({false,false,false}) => false
 *"
  input BooleanList in_booleanlist;
  output Boolean out_boolean;
protected 
  type BooleanList = list<Boolean>;
algorithm 
  out_boolean:=
  matchcontinue (in_booleanlist)
    local
      Boolean b,res;
      BooleanList rest;
    case ({b}) then b; 
    case (b :: rest)
      equation 
        equality(b = true); then true;
    case (b :: rest)
      equation 
        equality(b = false);
        res = bool_or_list(rest); then res;
  end matchcontinue;
end bool_or_list;

public function bool_and_list "* function: bool_and_list
 ** Takes a list of boolean values and applies the boolean 'and' operator on the elements
 ** For example,
 ** bool_and_list({true, true}) => true
 ** bool_and_list({false,false,true}) => false
 *"
  input BooleanList in_booleanlist;
  output Boolean out_boolean;
protected 
  type BooleanList = list<Boolean>;
algorithm 
  out_boolean:=
  matchcontinue (in_booleanlist)
    local
      Boolean b,res;
      BooleanList rest;
    case ({b}) then b; 
    case (b :: rest)
      equation 
        equality(b = false); then false;
    case (b :: rest)
      equation 
        equality(b = true);
        res = bool_and_list(rest); then res;
  end matchcontinue;
end bool_and_list;

public function bool_string "* function: bool_string
 ** Takes a boolean value and returns a string representation of the boolean value.
 ** For example,
 ** bool_string(true) => \"true\"
 *"
  input Boolean in_boolean;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_boolean)
    case true then "true"; 
    case false then "false"; 
  end matchcontinue;
end bool_string;

public function string_equal "* function: string_equal
 ** Takes two strings and returns true if the strings are equal
 ** For example,
 ** string_equal(\"a\",\"a\") => true
 *"
  input String in_string1;
  input String in_string2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_string1,in_string2)
    local String a,b;
    case (a,b)
      equation 
        equality(a = b); then true;
    case (_,_) then false; 
  end matchcontinue;
end string_equal;

public function list_matching "* function: list_matching
 ** For example,
 ** Takes a list of values and a matching function over the values and returns a
 ** sub list of values for which the matching function succeeds.
 ** For example,
 ** given function is_numeric(string) => ()  which succeeds if the string is numeric.
 ** list_matching({\"foo\",\"1\",\"bar\",\"4\"},is_numeric) => {\"1\",\"4\"}
 *"
  input VType_aList in_vtype_alist;
  input FuncType1 in_functype1;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
  function FuncType1
    input Type_a in_type_a;
  end FuncType1;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_alist,in_functype1)
    local
      VType_aList vl_1,vl;
      Type_a v;
      FuncType1 cond;
    case ({},_) then {}; 
    case (v :: vl,cond)
      equation 
        cond(v);
        vl_1 = list_matching(vl, cond); then v :: vl_1;
    case (v :: vl,cond)
      equation 
        failure(cond(v));
        vl_1 = list_matching(vl, cond); then vl_1;
  end matchcontinue;
end list_matching;

public function apply_option "* function: apply_option
 ** Takes an option value and a function over the value. It returns in another option value, resulting 
 ** from the application of the function on the value.
 ** For example,
 ** apply_option(SOME(1), int_string) => SOME(\"1\")
 ** apply_option(NONE, int_string) => NONE
 *"
  input VType_aOption in_vtype_aoption;
  input FuncType1 in_functype1;
  output VType_bOption out_vtype_boption;
protected 
  replaceable type Type_a;
  type VType_aOption = Option<Type_a>;
  function FuncType1
    input Type_a in_type_a;
    output Type_b out_type_b;
  protected 
    replaceable type Type_b;
  end FuncType1;
  replaceable type Type_b;
  type VType_bOption = Option<Type_b>;
algorithm 
  out_vtype_boption:=
  matchcontinue (in_vtype_aoption,in_functype1)
    local
      Type_b b;
      Type_a a;
      FuncType1 rel;
    case (NONE,_) then NONE; 
    case (SOME(a),rel)
      equation 
        b = rel(a); then SOME(b);
  end matchcontinue;
end apply_option;

public function make_option " function make_option
 ** Makes a value into value option, using SOME(value) 
*"
  input Type_a in_type_a;
  output VType_aOption out_vtype_aoption;
protected 
  replaceable type Type_a;
  type VType_aOption = Option<Type_a>;
algorithm 
  out_vtype_aoption:=
  matchcontinue (in_type_a)
    local Type_a v;
    case (v) then SOME(v); 
  end matchcontinue;
end make_option;

public function list_split "* function: list_split
 ** Takes a list of values and an position value.
 ** The function returns the list splitted into two lists at the position given as argument.
 ** For example,
 ** list_split({1,2,5,7},2) => ({1,2},{5,7})
 *"
  input VType_aList in_vtype_alist;
  input Integer in_integer;
  output VType_aList out_vtype_alist1;
  output VType_aList out_vtype_alist2;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  (out_vtype_alist1,out_vtype_alist2):=
  matchcontinue (in_vtype_alist,in_integer)
    local
      VType_aList a,b,c;
      Integer length,index;
    case (a,0) then ({},a); 
    case (a,index)
      equation 
        length = list_length(a);
        (index > length) = true;
        print(
          "Index out of bounds (greater than list length) in relation list_split\n"); then fail();
    case (a,index)
      equation 
        (index < 0) = true;
        print(
          "Index out of bounds (less than zero) in relation list_split\n"); then fail();
    case (a,index)
      equation 
        (index >= 0) = true;
        length = list_length(a);
        (index <= length) = true;
        (b,c) = list_split2(a, {}, index); then (c,b);
  end matchcontinue;
end list_split;

protected function list_split2 "* helper function to list_split
 *"
  input VType_aList in_vtype_alist1;
  input VType_aList in_vtype_alist2;
  input Integer in_integer3;
  output VType_aList out_vtype_alist1;
  output VType_aList out_vtype_alist2;
protected 
  replaceable type Type_a;
  type VType_aList = list<Type_a>;
algorithm 
  (out_vtype_alist1,out_vtype_alist2):=
  matchcontinue (in_vtype_alist1,in_vtype_alist2,in_integer3)
    local
      VType_aList a,b,c,d,rest;
      Integer index,new_index;
    case (a,b,index)
      equation 
        (index == 0) = true; then (a,b);
    case (a :: rest,b,index)
      local Type_a a;
      equation 
        new_index = index - 1;
        c = list_append(b, {a});
        (c,d) = list_split2(rest, c, new_index); then (c,d);
    case (_,_,_)
      equation 
        print("list_split2 failed\n"); then fail();
  end matchcontinue;
end list_split2;

public function int_positive "* function: int_positive
 ** Returns true if integer value is positive (>= 0)
 **
 *"
  input Integer v;
  output Boolean res;
algorithm 
  res := (v >= 0);
end int_positive;

public function option_to_list "* function: option_to_list
 ** Returns an empty list for NONE and a list containing
 ** the one element for SOME. To use with list_append
 *"
  input VType_aOption in_vtype_aoption;
  output VType_aList out_vtype_alist;
protected 
  replaceable type Type_a;
  type VType_aOption = Option<Type_a>;
  type VType_aList = list<Type_a>;
algorithm 
  out_vtype_alist:=
  matchcontinue (in_vtype_aoption)
    local Type_a e;
    case NONE then {}; 
    case SOME(e) then {e}; 
  end matchcontinue;
end option_to_list;

public function flatten_option "* function: flatten_option
 ** Returns the second argument if NONE and
 ** the contents of SOME if SOME
 *"
  input VType_aOption in_vtype_aoption;
  input Type_a in_type_a;
  output Type_a out_type_a;
protected 
  replaceable type Type_a;
  type VType_aOption = Option<Type_a>;
algorithm 
  out_type_a:=
  matchcontinue (in_vtype_aoption,in_type_a)
    local Type_a n,c;
    case (NONE,n) then n; 
    case (SOME(c),n) then c; 
  end matchcontinue;
end flatten_option;

public function is_empty_string
  input String in_string;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_string)
    case "" then true; 
    case _ then false; 
  end matchcontinue;
end is_empty_string;

public function is_not_empty_string
  input String in_string;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_string)
    case "" then false; 
    case _ then true; 
  end matchcontinue;
end is_not_empty_string;
end Util;

